# common emitter amplifier, sinusoidal output voltage
# all voltages normalized by Vcc
# all currents normalized by Vcc/R

from pylab import *

#initialization
np = 360 # reduce if the simulation is too slow
t = linspace(0, 1, np + 1)
pattern = empty(np + 1)
pattern[0 : np / 2 + 1] = 1
pattern[np / 2 + 1 : np + 1] = - 1

MceQ = 1.0 / 2.0 # you can adjust this, move the quiescent operating point from the optimal position
JcQ = 1 - MceQ

np = 50 # number of frames, reduce if the simulation is too slow

Mm = min([MceQ, 1 - MceQ])

m = empty(np + 1)
Pcc = empty(np + 1)
Pd = empty(np + 1)
Pr = empty(np + 1)
Prac = empty(np + 1)
efficiency = empty(np + 1)

close('all')
rc('text', usetex = True)
rc('font', family = 'serif')
rc('font', size = 12)
rcParams['text.latex.preamble'] = [r'\usepackage{amsmath}']

xt = linspace(0, 1, 5)

for k in range(np + 1):

    m[k] = float(k) / np # modulation index
    mce = MceQ + m[k] * Mm * pattern
    jc = 1 - mce
    mr = 1 - mce
    pcc = 1 * jc # 1 * is for educational purposes, a replacement for Vcc *
    pd = mce * jc
    pr = mr * jc
    prac = (jc - JcQ) * (mr - (1 - MceQ))
    Pcc[k] = mean(pcc)
    Pd[k] = mean(pd)
    Pr[k] = mean(pr)
    Prac[k] = mean(prac)
    efficiency[k] = Prac[k] / Pcc[k] * 100

    fig, axes = plt.subplots(nrows = 4, ncols=2, figsize = (12, 9))

    axes[0, 0].plot(t, mce)
    #axes[0, 0].set_xlabel(r'$t / T$')
    axes[0, 0].set_ylabel(r'$v_{CE} / V_{CC}$')
    axes[0, 0].set_xlim(0, 1)
    axes[0, 0].set_xticks(xt)
    axes[0, 0].set_ylim(0, 1)

    axes[1, 0].plot(t, jc)
    #axes[1, 0].set_xlabel(r'$t / T$')
    axes[1, 0].set_ylabel(r'$i_C / (V_{CC} / R)$')
    axes[1, 0].set_xlim(0, 1)
    axes[1, 0].set_xticks(xt)
    axes[1, 0].set_ylim(0, 1)

    axes[2, 0].plot(t, mr)
    #axes[2, 0].set_xlabel(r'$t / T$')
    axes[2, 0].set_ylabel(r'$v_R / V_{CC}$')
    axes[2, 0].set_xlim(0, 1)
    axes[2, 0].set_xticks(xt)
    axes[2, 0].set_ylim(0, 1)

    axes[3, 0].plot(mce, jc, '.')
    axes[3, 0].set_xlabel(r'$v_{CE} / V_{CC}$')
    axes[3, 0].set_ylabel(r'$i_C / (V_{CC} / R)$')
    axes[3, 0].set_xlim(0, 1)
    axes[3, 0].set_xticks([0, 0.25, 0.5, 0.75, 1])
    axes[3, 0].set_ylim(0, 1)

    axes[0, 1].plot(t, pcc)
    #axes[0, 1].set_xlabel(r'$t / T$')
    axes[0, 1].set_ylabel(r'$p_{CC} / (V_{CC}^2 / R)$')
    axes[0, 1].set_xlim(0, 1)
    axes[0, 1].set_xticks(xt)
    axes[0, 1].set_ylim(0, 1)

    axes[1, 1].plot(t, pd)
    #axes[1, 1].set_xlabel(r'$t / T$')
    axes[1, 1].set_ylabel(r'$p_D / (V_{CC}^2 / R)$')
    axes[1, 1].set_xlim(0, 1)
    axes[1, 1].set_xticks(xt)
    axes[1, 1].set_ylim(0, 1)

    axes[2, 1].plot(t, pr)
    #axes[2, 1].set_xlabel(r'$t / T$')
    axes[2, 1].set_ylabel(r'$p_R / (V_{CC}^2 / R)$')
    axes[2, 1].set_xlim(0, 1)
    axes[2, 1].set_xticks(xt)
    axes[2, 1].set_ylim(0, 1)

    axes[3, 1].plot(t, prac)
    axes[3, 1].set_xlabel(r'$t / T$')
    axes[3, 1].set_ylabel(r'$p_{Rac} / (V_{CC}^2 / R)$')
    axes[3, 1].set_xlim(0, 1)
    axes[3, 1].set_xticks(xt)
    axes[3, 1].set_ylim(0, 1)

    #savefig('ce1-{:03d}.png'.format(k), dpi = 600)#, bbox_inches = 'tight')
    savefig('ce2-{:03d}.pdf'.format(k))#, bbox_inches = 'tight')
    
    close(fig)

figure(2)
plot(m, Pcc, 'g', label = r'$P_{CC}$')
plot(m, Pd, 'b', label = r'$P_{D}$')
plot(m, Pr, 'r', label = r'$P_{R}$')
plot(m, Prac, 'y', label = r'$P_{R \, ac}$')
xlabel(r'$V_m / V_{m max}$')
ylabel(r'$P_{CC}, \; P_D, \; P_R, \; P_{Rac}, \; \text{all} \; / (V_{CC}^2/R)$')
xlim(0, 1)
ylim(0, 1.1*JcQ)
legend(loc = 'lower left')
savefig('ce2_powers.pdf', bbox_inches = 'tight')

figure(3)
plot(m, efficiency)
xlabel(r'$V_m / V_{m max}$')
ylabel(r'$\eta [\%]$')
ylim(0, 100)
savefig('ce2_efficiency.pdf', bbox_inches = 'tight')

